@@ -46,8 +46,12 @@ class Submission(object):
         self._abs_work_base = os.path.abspath(work_base)
 
         self.resources = resources
-        self.forward_common_files= forward_common_files
-        self.backward_common_files = backward_common_files
+        self.forward_common_files = sorted(forward_common_files) \
+            if isinstance(forward_common_files, list) \
+            else forward_common_files
+        self.backward_common_files = sorted(backward_common_files) \
+            if isinstance(backward_common_files, list) \
+            else backward_common_files
 
         self.submission_hash = None
         # warning: can not remote .copy() or there will be bugs
